'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

require('babel/polyfill');

var debug = (0, _debug2['default'])('service');

var Service = (function () {
  function Service(apiUrl, options) {
    _classCallCheck(this, Service);

    this.apiUrl = apiUrl;
    this.version = options.version || '*';
    this.userAgent = options.userAgent || 'DouglasLab API Wrapper';
    this.returnPromise = options.returnPromise || false;
    this.request = _superagent2['default'];
    debug('initializing wrapper: url: %s, path: %s, ua %s', this.apiUrl, this.version, this.userAgent);
  }

  _createClass(Service, [{
    key: '_generateAuthorizationHeader',
    value: function _generateAuthorizationHeader(apiKey, apiSecret) {
      var timestamp = parseInt(Date.now() / 1000, 10);
      var hmac = _crypto2['default'].createHmac('sha1', apiSecret).update(apiKey).digest('hex');
      var token = _crypto2['default'].createHash('md5').update(hmac + timestamp).digest('hex');
      return 'key=' + apiKey + ', token=' + token + ', ts=' + timestamp;
    }
  }, {
    key: '_request',
    value: function _request(user, method, path, payload, callback) {
      var agent = this.request(method, this.apiUrl + path).set(this.generateHeaders(user));
      if (payload) {
        agent.send(payload);
      }
      if (this.returnPromise) {
        return new Promise(function (resolve, reject) {
          agent.end(function (err, res) {
            if (err) {
              reject(res ? res.error : err);
            } else {
              resolve(res.body);
            }
          });
        });
      } else {
        agent.end(this.handleResult(callback));
      }
    }
  }, {
    key: 'generateHeaders',
    value: function generateHeaders(user, overrideContentType) {
      var headers = {
        'Accept-Version': this.version,
        'User-Agent': this.userAgent
      };
      if (overrideContentType) {
        Object.assign(headers, overrideContentType);
      } else {
        Object.assign(headers, { 'Accept': 'application/json' }, { 'Content-Type': 'application/json' });
      }
      if (user) {
        headers['X-API-Authorization'] = this._generateAuthorizationHeader(user.apiKey, user.apiSecret);
      }
      return headers;
    }
  }, {
    key: 'handleResult',
    value: function handleResult(cb) {
      return (function (err, res) {
        if (err && err.code && err.code.includes('ECONNREFUSED')) {
          console.error('cannot contact API at ' + this.apiUrl);
          this.callback(err, { error: true, data: 'cannot contact API at ' + this.apiUrl });
        } else {
          this.callback(null, res.body);
        }
      }).bind({ callback: cb, apiUrl: this.apiUrl });
    }
  }, {
    key: 'handleSpecialCall',
    value: function handleSpecialCall(agent, callback) {
      var _this = this;

      if (this.returnPromise) {
        return new Promise(function (resolve, reject) {
          agent.end(function (err, res) {
            if (err) {
              if (err.code && err.code.includes('ECONNREFUSED')) {
                console.error('cannot contact API at ' + _this.apiUrl);
                reject({ error: true, data: 'cannot contact API at ' + _this.apiUrl });
              } else {
                reject(res ? res.error : err);
              }
            } else {
              resolve(res.body);
            }
          });
        });
      } else {
        agent.end(this.handleResult(callback));
      }
    }
  }, {
    key: 'get',
    value: function get(user, path, callback) {
      return this._request(user, 'GET', path, null, callback);
    }
  }, {
    key: 'post',
    value: function post(user, path, body, callback) {
      return this._request(user, 'POST', path, body, callback);
    }
  }, {
    key: 'put',
    value: function put(user, path, body, callback) {
      return this._request(user, 'PUT', path, body, callback);
    }
  }, {
    key: 'del',
    value: function del(user, path, callback) {
      return this._request(user, 'DELETE', path, null, callback);
    }
  }]);

  return Service;
})();

exports['default'] = Service;
module.exports = exports['default'];